@@ -14,11 +14,15 @@ import os
 import datetime
 import pandas as pd
 from meteostat.core import Core
+from meteostat import units
 from math import floor, nan
 from copy import copy
 
 class Hourly(Core):
 
+  # The cache subdirectory
+  _cache_subdir = 'hourly'
+
   # The list of weather Stations
   _stations = None
 
@@ -48,12 +52,25 @@ class Hourly(Core):
     'coco'
   ]
 
+  # Data tapes
+  _types = {
+    'temp': 'float64',
+    'dwpt': 'float64',
+    'rhum': 'float64',
+    'prcp': 'float64',
+    'snow': 'float64',
+    'wdir': 'float64',
+    'wspd': 'float64',
+    'wpgt': 'float64',
+    'tsun': 'float64',
+    'coco': 'float64'
+  }
+
   # Columns for date parsing
   _parse_dates = { 'time': [0, 1] }
 
   # Default aggregation functions
   _aggregations = {
-    'time': 'first',
     'temp': 'mean',
     'dwpt': 'mean',
     'rhum': 'mean',
@@ -83,9 +100,11 @@ class Hourly(Core):
               for file in files:
 
                   if os.path.isfile(file['path']) and os.path.getsize(file['path']) > 0:
-                      df = pd.read_feather(file['path'])
 
-                      self._data = self._data.append(df[(df['time'] >= self._start) & (df['time'] <= self._end)])
+                      df = pd.read_parquet(file['path'])
+
+                      time = df.index.get_level_values('time')
+                      self._data = df.loc[(time >= self._start) & (time <= self._end)]
 
   def __init__(
     self,
@@ -113,6 +132,9 @@ class Hourly(Core):
       if isinstance(stations, pd.DataFrame):
           self._stations = stations
       else:
+          if not isinstance(stations, list):
+              stations = [stations]
+
           self._stations = pd.DataFrame(stations, columns = ['id'])
 
       # Set start date
@@ -127,80 +149,83 @@ class Hourly(Core):
       except:
           raise Exception('Cannot read hourly data')
 
+      # Clear cache
+      self.clear_cache()
+
   def normalize(self):
 
       # Create temporal instance
-      self._temp = copy(self)
-
-      # List of columns
-      columns = ['station', 'time']
-
-      # Dynamically append columns
-      for column in self._temp._columns[2:]:
-          columns.append(column)
+      temp = copy(self)
 
       # Create result DataFrame
-      result = pd.DataFrame(columns = columns)
+      result = pd.DataFrame(columns = temp._columns[2:])
 
       # Go through list of weather stations
-      for station in self._temp._stations['id'].tolist():
+      for station in temp._stations['id'].tolist():
           # Create data frame
-          df = pd.DataFrame(columns = columns)
+          df = pd.DataFrame(columns = temp._columns[2:])
           # Add time series
-          df['time'] = pd.date_range(self._temp._start, self._temp._end, freq='1H')
+          df['time'] = pd.date_range(temp._start, temp._end, freq = '1H')
           # Add station ID
           df['station'] = station
           # Add columns
-          for column in columns[2:]:
+          for column in temp._columns[2:]:
               # Add column to DataFrame
               df[column] = nan
 
           result = pd.concat([result, df], axis = 0)
 
+      # Set index
+      result = result.set_index(['station', 'time'])
+
       # Merge data
-      self._temp._data = pd.concat([self._temp._data, result], axis = 0).groupby(['station', 'time'], as_index = False).first()
+      temp._data = pd.concat([temp._data, result], axis = 0).groupby(['station', 'time'], as_index = True).first()
 
       # Return class instance
-      try:
-          return self._temp
-      finally:
-          self._temp = None
+      return temp
 
   def interpolate(self, limit = 3):
 
       # Create temporal instance
-      self._temp = copy(self)
+      temp = copy(self)
 
       # Apply interpolation
-      self._temp._data = self._temp._data.groupby('station').apply(lambda group: group.interpolate(method = 'linear', limit = limit, limit_direction = 'both', axis = 0))
+      temp._data = temp._data.groupby(level = 'station').apply(lambda group: group.interpolate(method = 'linear', limit = limit, limit_direction = 'both', axis = 0))
 
       # Return class instance
-      try:
-          return self._temp
-      finally:
-          self._temp = None
+      return temp
 
   def aggregate(self, freq = None, functions = None, spatial = False):
 
       # Create temporal instance
-      self._temp = copy(self)
+      temp = copy(self)
 
       # Update default aggregations
       if functions is not None:
-          self._temp._aggregations.update(functions)
+          temp._aggregations.update(functions)
 
       # Time aggregation
-      self._temp._data = self._temp._data.groupby(['station', pd.Grouper(key = 'time', freq = freq)]).agg(self._temp._aggregations)
+      temp._data = temp._data.groupby(['station', pd.Grouper(level = 'time', freq = freq)]).agg(temp._aggregations)
 
       # Spatial aggregation
       if spatial:
-          self._temp._data = self._temp._data.groupby([pd.Grouper(key = 'time', freq = freq)]).mean()
+          temp._data = temp._data.groupby([pd.Grouper(key = 'time', freq = freq)]).mean()
 
       # Return class instance
-      try:
-          return self._temp
-      finally:
-          self._temp = None
+      return temp
+
+  def convert(self, units):
+
+      # Create temporal instance
+      temp = copy(self)
+
+      # Change data units
+      for parameter, unit in units.items():
+
+          temp._data[parameter] = temp._data[parameter].apply(unit)
+
+      # Return class instance
+      return temp
 
   def coverage(self, parameter = None):
 
@@ -218,5 +243,12 @@ class Hourly(Core):
 
   def fetch(self):
 
+      # Copy DataFrame
+      temp = copy(self._data)
+
+      # Remove station index if it's a single station
+      if len(self._stations.index) == 1 and 'station' in temp.index.names:
+          temp = temp.reset_index(level = 'station', drop = True)
+
       # Return data frame
-      return copy(self._data)
+      return temp
